SystemOrganization addCategory: #LostInMorvanLPC!
SystemOrganization addCategory: 'LostInMorvanLPC-Tests'!

TestCase subclass: #LPCSpriteTest
	instanceVariableNames: 'spritePNGFile sprite'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanLPC-Tests'!

!LPCSpriteTest commentStamp: 'FrancoisJarriges 5/20/2016 14:34' prior: 0!
I represent a TestCase for LPCSprite. The test is built on a demo sprite created at setUp by invoking :

spritePNGFile := FileSystem workingDirectory / 'lostInmorvan.png'.
sprite := LPCSprite fromUniversalLPLCSpritePNGFile: spritePNGFile.

This PNG file has been dowloaded from from http://
gaurav.munjal.us/Universal-LPC-Spritesheet-Character-Generator/

The test is based on :
	checking sprite state variables at initialization and after each actions  request messages
	actions request protocol check
	corresponding animations  content check (Form collections size, Form elements class)
	
Instance Variables
	sprite:			<LPCSprite>
	spritePNGFile:	<FileReference >

!

!LPCSpriteTest methodsFor: 'running' stamp: 'FrancoisJarriges 5/20/2016 16:20'!
setUp

	spritePNGFile := FileSystem workingDirectory / 'lostInmorvan.png'.
	sprite := LPCSprite fromUniversalLPCSpritePNGFile: spritePNGFile.
! !

!LPCSpriteTest methodsFor: 'running' stamp: 'FrancoisJarriges 5/18/2016 07:19'!
tearDown

		sprite := nil.! !

!LPCSpriteTest methodsFor: 'tests' stamp: 'FrancoisJarriges 5/20/2016 13:10'!
testFromUniversalLPLCSpritePNGFile

	self assert: ((sprite tileSize / 16) isInteger and: sprite tileSize > 0).
	self assert: sprite posture isNil.
	self assert: sprite action isNil.
	self assert: sprite currentAnimation isNil.
	
	sprite act: #spellcast.
	self assert: sprite posture isNil.
	self assert: sprite action isNil.
	self assert: sprite currentAnimation isNil.
	
	sprite show: #front.
	self assert: sprite posture == #front.
	self assert: sprite action isNil.
	self assert: sprite currentAnimation isNil.
	
	sprite act: #spellcast.
	self assert: sprite posture == #front.
	self assert: sprite action == #spellcast.
	self assert: sprite currentAnimation class == OrderedCollection.
	self assert: sprite currentAnimation anyOne class == Form.
	self assert: sprite currentAnimation size = 7.
	
	sprite act: #walk.
	self assert: sprite posture == #front.
	self assert: sprite action == #walk.
	self assert: sprite currentAnimation class == OrderedCollection.
	self assert: sprite currentAnimation anyOne class == Form.
	self assert: sprite currentAnimation size = 9.
		
	sprite show: #right.
	self assert: sprite posture == #right.
	self assert: sprite action == #walk.
	self assert: sprite currentAnimation class == OrderedCollection.
	self assert: sprite currentAnimation anyOne class == Form.
	self assert: sprite currentAnimation size = 9.
	
	sprite act: #hurt.
	self assert: sprite posture == #right.
	self assert: sprite action == #hurt.
	self assert: sprite currentAnimation class == OrderedCollection.
	self assert: sprite currentAnimation size = 0.


	! !

BorderedMorph subclass: #LPCWorld
	instanceVariableNames: 'sprites'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanLPC'!

LPCWorld class
	instanceVariableNames: 'uniqueInstance'!

!LPCWorld commentStamp: 'FrancoisJarriges 5/20/2016 15:24' prior: 0!
I represent a  world for a Liberated Pixel Cup (LPC) style game where sprites can move and act.

For the Responsibility part: Three sentences about my main responsibilities - what I do, what I know.

For the Collaborators Part: State my main collaborators and one line about how I interact with them. 

Public API and Key Messages

- message one   
- message two 
- (for bonus points) how to create instances.

   One simple example is simply gorgeous.
 
Internal Representation and Key Implementation Points.


    Implementation Points!

LPCWorld class
	instanceVariableNames: 'uniqueInstance'!

!LPCWorld class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/22/2016 11:41'!
createGameFromSpritePNGFiles:  aFileNameCollection

	1 to: aFileNameCollection size do: [ :eachFileName | 
		| eachSprite eachSpriteName eachFileReference|
		
		eachSpriteName:= eachFileName withoutPeriodSuffix.
		eachFileReference := FileSystem workingDirectory / eachFileName.
		eachSprite := LPCSprite fromUniversalLPCSpritePNGFile: eachFileReference.
		
		self notYetImplemented .
		]! !

!LPCWorld class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 16:10'!
current
	
	^ uniqueInstance ifNil: [ uniqueInstance := self new ]! !

!LPCWorld class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 16:12'!
initialize

	self reset! !

LPCWorld initialize!

!LPCWorld class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 16:12'!
reset

	uniqueInstance := nil! !

!LPCWorld methodsFor: 'as yet unclassified' stamp: 'FrancoisJarriges 5/20/2016 19:27'!
initialize
	
	super initialize.
	
	self borderWidth: 2.
	self color: (Color fromString: '181818').
	bounds := 0@0 corner: 960@576.
	
	sprites := Dictionary new.
	! !

Object subclass: #LPCSprite
	instanceVariableNames: 'tileSize posture action animations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanLPC'!

LPCSprite class
	instanceVariableNames: 'actions postures universalLPCSpritePNGFormat'!

!LPCSprite commentStamp: 'FrancoisJarriges 5/22/2016 11:27' prior: 0!
I represent a sprite in the Liberated Pixel Cup style (LPC)

I provide an instance-creation method to build the animation sequences  for a new sprite from a PNG file (as got from http://
gaurav.munjal.us/Universal-LPC-Spritesheet-Character-Generator/ ) 

The syntax is like this :

mySprite := LPCSprite fromUniversalLPLCSpritePNGFile: FileSystem workingDirectory / 'lostInmorvan.png'.

LPCSprite class provide three configuration tables :

	postures 
		all possible postures for a LPCSprite  as an OrderedCollection of symbols
		
	actions 
		all possible actions for a LPCSprite  as an OrderedCollection of symbols
		
	universalLPCSpritePNGFormat
		the map of postures implemented for each action in the source PNG file as an OrderedCollection of associations

Once created, this sprite has to receive a first posture request message before treating others posture or action requests. 
This actions protocol is like that :

	mySprite show: #front .
	mySprite act: #walk.

I provide these accessing methods  :
	mySprite posture .   			-> returns #front
	mySprite action.	 	 		-> returns #walk
	mySprite currentAnimation. 	-> returns an OrderedCollection(Form(64x64x32) ...)
	mySprite tileSize.				-> returns 64 
	mySprite animationForAction: #hurt  inPosture: #back 
								-> returns an OrderedCollection() 
	
 Internal Representation and Key Implementation Points.

    Instance Variables
	tileSize 							<SmallInteger>
	posture  							<Symbol>
	action  							<Symbol>
	animations 						<Dictionary>
		
    Class Instance Variables 
	initialized 						<Boolean>
	actions 							<OrderedCollection>
	postures 						<OrderedCollection>	
	universalLPCSpritePNGFormat 	<OrderedCollection>
		
    Implementation Points
	an animation is represented as an OrderedCollection of poses
	each pose is an image represented as a square Form which has the size of the sprite tile (tileSize))!

LPCSprite class
	instanceVariableNames: 'actions postures universalLPCSpritePNGFormat'!

!LPCSprite class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 08:19'!
actions
	^ actions! !

!LPCSprite class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 5/21/2016 08:36'!
fromUniversalLPCSpritePNGFile: aFileReference

	| newSprite spriteSheet stripRank |
	
	newSprite := self initialize new.
	
	spriteSheet := aFileReference binaryReadStreamDo: [ :stream | 
		PNGReadWriter formFromStream: stream ].
	
	newSprite tileSize: spriteSheet height / universalLPCSpritePNGFormat size.
	
	stripRank := 0.
	
	0 to: spriteSheet height - 1 by: newSprite tileSize do: [ :animationOffset | 
		
		| animationStrip animation |
		
		animationStrip := spriteSheet copy: 
			(0 @ animationOffset corner: spriteSheet width @ (animationOffset + newSprite tileSize)).
		
		stripRank := stripRank + 1.
			
		animationStrip primCountBits > 0 ifTrue: [ 
			
			| action posture |
			
			action := (universalLPCSpritePNGFormat at: stripRank) key.
			posture := (universalLPCSpritePNGFormat at: stripRank) value.
			animation  := newSprite animationForAction: action
					inPosture: posture.
			
			0 to: spriteSheet width - 1 by: newSprite tileSize do: [ :poseOffset |
				
				| pose |
				
				pose := animationStrip copy:
					(poseOffset @ 0 corner: (poseOffset + newSprite tileSize) @ newSprite tileSize).
					
				pose primCountBits > 0 ifTrue: [ 
					animation add: pose ].
				].		
			].
		].
	^ newSprite 
	
	
! !

!LPCSprite class methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/22/2016 11:25'!
initialize

	super initialize.
	
	postures := OrderedCollection new 
		add: #back;
		add: #left;
		add: #front;
		add: #right;
		yourself.
	
	actions := OrderedCollection new
		add: #spellcast;
		add: #thrust;
		add: #walk;
		add: #slash;
		add: #shoot;
		add: #hurt;
		yourself.
		
	universalLPCSpritePNGFormat := OrderedCollection new
	
		add: #spellcast 	-> #back;
		add: #spellcast 	-> #left;
		add: #spellcast 	-> #front;
		add: #spellcast 	-> #right;
		
		add: #thrust 		-> #back;
		add: #thrust 		-> #left;
		add: #thrust 		-> #front;
		add: #thrust 		-> #right;
		
		add: #walk 			-> #back;
		add: #walk 			-> #left;
		add: #walk 			-> #front; 
		add: #walk 			-> #right; 
		
		add: #slash 			-> #back; 
		add: #slash 			-> #left;
		add: #slash 			-> #front;
		add: #slash 			-> #right;
		
		add: #shoot 			-> #back;
		add: #shoot 			-> #left;
		add: #shoot 			-> #front;
		add: #shoot 			-> #right; 
		
		add: #hurt 			-> #front; 
		
		yourself.


! !

LPCSprite initialize!

!LPCSprite class methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 10:13'!
postures
	^ postures! !

!LPCSprite methodsFor: 'actions' stamp: 'FrancoisJarriges 5/22/2016 11:34'!
act: anAction

	((self class actions includes: anAction) and: posture isNotNil) ifTrue: [ 
		action := anAction ]! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 12:32'!
action
	^ action! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/22/2016 11:37'!
animationForAction: anAction inPosture: aPosture

	((self class actions includes: anAction) and:
		(self class postures includes: aPosture)) ifFalse: [ ^ nil ].
		
	^ (animations at: anAction) at: aPosture.

! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 13:26'!
currentAnimation

	(posture isNil or: action isNil) ifTrue: [ ^ nil ].
		
	^ (animations at: action) at: posture.

! !

!LPCSprite methodsFor: 'initialization' stamp: 'FrancoisJarriges 5/22/2016 11:30'!
initialize

	super initialize.
	
	animations := Dictionary new.
	1 to: self class actions size do: [  :eachAction | 
		
		| actionKey |
		
		actionKey := self class actions at: eachAction.
		animations at: actionKey put: Dictionary new.
		
		1 to: self class postures size do: [ :eachPosture |
			(animations at: actionKey) 
				at: (self class postures at: eachPosture)
				put: OrderedCollection new.
			]	
		]
	! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 12:32'!
posture
	^ posture! !

!LPCSprite methodsFor: 'actions' stamp: 'FrancoisJarriges 5/22/2016 11:37'!
show: aPosture

	(self class postures includes: aPosture) ifTrue: [ 
		posture := aPosture ]! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 11:29'!
tileSize

	^ tileSize ! !

!LPCSprite methodsFor: 'accessing' stamp: 'FrancoisJarriges 5/20/2016 11:37'!
tileSize: anInteger

	tileSize := anInteger! !

LPCWorld initialize!

LPCSprite initialize!
